require 'powertrack_v2/http'
require 'powertrack_v2/errors'
require 'multi_json'

module PowerTrackV2
  class Rule
    MAX_TAG_LENGTH = 255
    MAX_VALUE_LENGTH = 2048

    attr_accessor :value, :tag

    def initialize(value, tag=nil)
      @value = value
      @tag = tag
    end

    def as_json
      hash = {
        value: @value
      }
      hash[:tag] = @tag if @tag && !@tag.empty?
      hash
    end

    def valid?
      return false if @value.nil? || @value.length > MAX_VALUE_LENGTH
      return false if @tag && @tag.length > MAX_TAG_LENGTH
      true
    end

    def self.url(publisher, account, stream_label)
      "https://gnip-api.#{publisher}.com/rules/powertrack/accounts/#{account}/publishers/#{publisher}/#{stream_label}.json"
    end

    def self.get(url, optional_headers={})
      headers = PowerTrackV2::HTTP.headers(optional_headers)
      response = PowerTrackV2::HTTP.get(url, headers)
      handle_response(response)
    end

    def self.handle_response(response)
      return response.body if response.status == 200 || response.status < 300
      return PowerTrackV2::Errors.raise_error_for_code(response.status, response.body)
    end

    DEFAULT_POST_HEADERS = {
      'Content-Type' => 'application/json'
    }

    def self.post(url, rules, optional_headers={})
      validate(rules)

      base_headers = DEFAULT_POST_HEADERS.merge(optional_headers)
      headers = PowerTrackV2::HTTP.headers(base_headers)

      body = prepare_rules(rules)

      response = PowerTrackV2::HTTP.post(url, body, headers)
      handle_response(response)
    end

    def self.delete(url, rules, optional_headers={})
      post(url + '?_method=delete', rules, optional_headers)
    end

    def self.validate(rules)
      raise ::PowerTrackV2::Errors::TooManyRulesError if rules.length > 5000
      raise ::PowerTrackV2::Errors::InvalidRuleError unless rules.all? &:valid?
    end

    private

    def self.prepare_rules(rules)
      rules = rules.map {|rule| rule.as_json}
      ::MultiJson.dump({rules: rules})
    end
  end
end
